using System;
using System.Windows.Forms;
//using System.Collections.Generic;
//using System.Text;
using System.Runtime.InteropServices;

using MediaPortal.GUI.Library;
using MediaPortal.Profile;
using MediaPortal.Configuration;
using MediaPortal.Player;

namespace MonitorPowerControl
{
	public class MonitorPowerControl : MediaPortal.GUI.Library.IPlugin, MediaPortal.GUI.Library.ISetupForm
  {
    #region Variables
    //the name of the configuartion file
    const string configFile = "MonitorPowerControl.xml";
		
    //if true, then keep monitor on during music playing
    bool onMusic = true;

    //if true, then keep monitor on during radio playing
    bool onRadio = true;

    //if true, then keep monitor on during recording
    bool onRecording = true;

    //if true, then keep monitor on during TV playing
    bool onTV = true;

    //if true, then keep monitor on during video playing 
    bool onVideo = true;

    #endregion

    #region IPlugin Members

    /// <summary>
    /// Called by mediaportal to start your process plugin
		/// </summary>
    public void Start()
		{
			//set up event handlers
      g_Player.PlayBackStarted += new g_Player.StartedHandler(g_Player_PlayBackStarted);
			g_Player.PlayBackStopped += new g_Player.StoppedHandler(g_Player_PlayBackStopped);
			g_Player.PlayBackEnded += new g_Player.EndedHandler(g_Player_PlayBackEnded);

      loadConfig();
		}

		/// <summary>
    /// Called by mediaportal to stop your process plugin
		/// </summary>
    public void Stop()
		{
		}

		#endregion

		#region ISetupForm Members

    /// <summary>
    /// Returns the name of the plugin which is shown in the plugin menu
    /// </summary>
    /// <returns></returns>
		public string PluginName()
		{
			return "Monitor Power Control";
		}

    /// <summary>
    /// Returns the description of the plugin is shown in the plugin menu
    /// </summary>
    /// <returns></returns>
		public string Description()
		{
			return "Prevents the monitor from powering down during playback regardless of Windows Power Saving Settings.";
		}

		/// <summary>
    /// Returns the author of the plugin which is shown in the plugin menu
		/// </summary>
		/// <returns></returns>
    public string Author()
		{
			return "thFrgttn";
		}

		/// <summary>
    /// show the setup dialog and update media types if needed
		/// </summary>
    public void ShowPlugin()
		{
      loadConfig();
      MediaTypes mediaTypes = new MediaTypes(onMusic, onRadio, onRecording, onTV, onVideo);
      if (mediaTypes.ShowDialog() == DialogResult.OK) {
        bool changed = false;
        if (onMusic != mediaTypes.isMusicChecked()) {
          changed = true;
          onMusic = mediaTypes.isMusicChecked();
        }
        if (onRadio != mediaTypes.isRadioChecked()) {
          changed = true;
          onRadio = mediaTypes.isRadioChecked();
        }
        if (onRecording != mediaTypes.isRecordingChecked()) {
          changed = true;
          onRecording = mediaTypes.isRecordingChecked();
        }
        if (onTV != mediaTypes.isTVChecked()) {
          changed = true;
          onTV = mediaTypes.isTVChecked();
        }
        if (onVideo != mediaTypes.isVideoChecked()) {
          changed = true;
          onVideo = mediaTypes.isVideoChecked();
        }
        if (changed) {
          saveConfig();
        }
      }
		}

		/// <summary>
    /// Indicates whether plugin can be enabled/disabled
		/// </summary>
		/// <returns></returns>
    public bool CanEnable()
		{
			return true;
		}

		/// <summary>
    /// get ID of windowplugin belonging to this setup
		/// </summary>
		/// <returns></returns>
    public int GetWindowId()
		{
			return 770417;
		}

		/// <summary>
    /// Indicates if plugin is enabled by default
		/// </summary>
		/// <returns></returns>
    public bool DefaultEnabled()
		{
			return true;
		}

		/// <summary>
    /// indicates if a plugin has its own setup screen
		/// </summary>
		/// <returns></returns>
    public bool HasSetup()
		{
			return true;
		}

    /// <summary>
    /// If the plugin should have its own button on the home menu of Mediaportal then it
    /// should return true to this method, otherwise if it should not be on home
    /// it should return false
    /// </summary>
    /// <param name="strButtonText">text the button should have</param>
    /// <param name="strButtonImage">image for the button, or empty for default</param>
    /// <param name="strButtonImageFocus">image for the button, or empty for default</param>
    /// <param name="strPictureImage">subpicture for the button or empty for none</param>
    /// <returns>true  : plugin needs its own button on home
    ///          false : plugin does not need its own button on home</returns>
    public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
		{
			strButtonText = String.Empty;
      strButtonImage = String.Empty;
      strButtonImageFocus = String.Empty;
      strPictureImage = String.Empty;
      return false;
		}

		#endregion

		#region eventHandlers

		/// <summary>
		/// Handles the g_player PlayBackEnded Event
		/// </summary>
		/// <param name="type">the media type being played</param>
		/// <param name="filename">the filename of the media being played</param>
    void g_Player_PlayBackEnded(g_Player.MediaType type, string filename)
		{
			allowMonitorPowerOff();
		}

		/// <summary>
    /// Handles the g_player PlayBackStopped Event
		/// </summary>
		/// <param name="type"></param>
    /// <param name="type">the media type being played</param>
    /// <param name="filename">the filename of the media being played</param>
    void g_Player_PlayBackStopped(g_Player.MediaType type, int stoptime, string filename)
		{
			allowMonitorPowerOff();
		}

		/// <summary>
    /// Handles the g_player PlayBackStarted Event
		/// </summary>
    /// <param name="type">the media type being played</param>
    /// <param name="filename">the filename of the media being played</param>
    void g_Player_PlayBackStarted(g_Player.MediaType type, string filename)
		{
      if (onWithType(type)) {
        preventMonitorPowerOff();
      }
		}

		#endregion

		#region Kernel32 Importing
		
    /// <summary>
    /// The EXECUTION_STATE enum matches possible parameters and return values of SetThreadExecutionState
    /// </summary>
    [FlagsAttribute]
    protected enum EXECUTION_STATE : uint
		{
			ES_SYSTEM_REQUIRED = 0x00000001,
			ES_DISPLAY_REQUIRED = 0x00000002,
			// Legacy flag, should not be used.
			// ES_USER_PRESENT   = 0x00000004,
			ES_CONTINUOUS = 0x80000000,
		}

		/// <summary>
    /// Import SetThreadExecutionState from Kernel32.dll and matchup the EXECUTION_STATE enum
		/// </summary>
		/// <param name="state"></param>
		/// <returns></returns>
    [DllImport("Kernel32.DLL", CharSet = CharSet.Auto, SetLastError = true)]
    protected extern static EXECUTION_STATE SetThreadExecutionState(EXECUTION_STATE state);

		#endregion

    #region protected methods

    /// <summary>
    /// Tests if the monitor should stay on for the given MediaType
    /// </summary>
    /// <param name="type">the MediaType to test</param>
    /// <returns>true if the monitor should remain on for the given type</returns>
    protected bool onWithType(MediaPortal.Player.g_Player.MediaType type)
		{
			switch (type) {
				case g_Player.MediaType.Music:
					return onMusic;
				case g_Player.MediaType.Radio:
					return onRadio;
				case g_Player.MediaType.Recording:
					return onRecording;
				case g_Player.MediaType.TV:
					return onTV;
				case g_Player.MediaType.Video:
					return onVideo;
			}
      return false;
		}

		/// <summary>
		/// tells windows it can't turn the monitor off
		/// </summary>
    protected void preventMonitorPowerOff()
		{
			SetThreadExecutionState(EXECUTION_STATE.ES_DISPLAY_REQUIRED | EXECUTION_STATE.ES_CONTINUOUS);
		}

    /// <summary>
    /// tells windows it can turn the monitor off
    /// </summary>
    protected void allowMonitorPowerOff()
		{
			SetThreadExecutionState(EXECUTION_STATE.ES_CONTINUOUS);
    }

    /// <summary>
    /// loads the configuration
    /// </summary>
    protected void loadConfig()
    {
      //read the configuartion settings
      try {
        Settings reader = new Settings(Config.GetFile(Config.Dir.Config, configFile));
        onMusic = reader.GetValueAsBool("MediaTypes", "Music", true);
        onRadio = reader.GetValueAsBool("MediaTypes", "Radio", true);
        onRecording = reader.GetValueAsBool("MediaTypes", "Recording", true);
        onTV = reader.GetValueAsBool("MediaTypes", "TV", true);
        onVideo = reader.GetValueAsBool("MediaTypes", "Video", true);
      } catch {
        //file didn't exist - don't care
      }
    }

    /// <summary>
    /// saves the configuration
    /// </summary>
    protected void saveConfig()
    {
      //read the configuartion settings
      Settings writer = new Settings(Config.GetFile(Config.Dir.Config, configFile));
      writer.SetValueAsBool("MediaTypes", "Music", onMusic);
      writer.SetValueAsBool("MediaTypes", "Radio", onRadio);
      writer.SetValueAsBool("MediaTypes", "Recording", onRecording);
      writer.SetValueAsBool("MediaTypes", "TV", onTV);
      writer.SetValueAsBool("MediaTypes", "Video", onVideo);
    }

    #endregion
  }
}
